/**************************************************************************
* DSemu: Memory management and handling (mmu.c)                           *
* Released under the terms of the BSD Public Licence                      *
* Imran Nazar (tf@oopsilon.com), 2004                                     *
**************************************************************************/

#include <stdio.h>
#include "vtbl.h"
#include "arm.h"
#include "err.h"
#include "dsmmumain.h"
#include "dma.h"
#include "dsioreg.h"
#include "gpu.h"
#include "unzip.h"

//#define MMUMainDEBUG
//#define BIOSsize 1024

extern u16 *VRAM;
extern u8  *VRAM8;
extern RAMWORD *EWRAM, *IWRAM;
RAMWORD *GameROM, *BIOS9;

extern ARMREGS arm9reg;

extern u16 OAM[512];
extern u32 LCDcolour[32768];
extern u16 GPUPAL[1024];
extern u8 *GPUPAL8;

RAMWORD MMUMainEmpty;
u32 MMUMainROMsize; char *MMUMainROMfile;
u32 MMUMainBIOSsize;

u32 VRAMmap[553];

RAMWORD DTCM[4096], ITCM[4096];

int MMUMaininit(char *file)
{
    FILE *fp; u8 c; int a;

    MMUMainROMfile=file;

    EWRAM=(RAMWORD*)malloc(1048576*sizeof(RAMWORD));
    if(!EWRAM) RETFAIL("FAIL: MMU9: EWRAM allocation.");

    IWRAM=(RAMWORD*)malloc(8192*sizeof(RAMWORD));
    if(!IWRAM) RETFAIL("FAIL: MMU9: IWRAM allocation.");

    MMUMainEmpty.data=0xFFFFFFFF;
    MMUMainEmpty.op=ARM9opUNL;
    MMUMainEmpty.cond=ARM9condAL;

    RETPASS("MMU9: Memoryspace allocation complete.");
}

int MMUMainreset()
{
    HANDLE hFile; u8 c; int a,done=0,bytesread; u8 *tempROM;
    char BIOSpath[1024], INIpath[1024], ARM9name[1024], str[256], strout[384];
    int pathsize;
    HZIP hz; ZIPENTRY ze; int cnt;
    u32 ARM9size, ARM9source=0x08000000, ARM9dest=0x02004000, ARM9start=0x02004000;

    logvt->append("MMU9: Resetting.");

    memset(EWRAM,0,0x3FFFFF);
    memset(IWRAM,0,0x7FFF);
    memset(DTCM,0,4096*sizeof(RAMWORD));
    memset(ITCM,0,4096*sizeof(RAMWORD));
    logvt->append("MMU9: Memory zeroed.");

    pathsize=strstr(logvt->file,"\\log.txt")-logvt->file;
    strncpy(BIOSpath, logvt->file, pathsize); BIOSpath[pathsize]=0;
    sprintf(INIpath, "%s\\dsemu.ini", BIOSpath);
    GetPrivateProfileString("General","BIOS9","bioshack9.bin",str,256,INIpath);
    sprintf(BIOSpath, "%s\\%s", BIOSpath,str);

    if(!strncmp((MMUMainROMfile+strlen(MMUMainROMfile)-3),"zip",3))
    {
        hz=OpenZipFromName(MMUMainROMfile,0);
        if(!hz) RETFAIL("FAIL: MMU9: Zip file open.");
        #ifdef MMUMainDEBUG
        logvt->append("MMU9: Zip file open.");
        #endif
        GetZipItem(hz,-1,&ze); cnt=ze.index; a=0;
        do {
            GetZipItem(hz,a,&ze);
            if(ze.unc_size && (strstr(ze.name,".gba") || strstr(ze.name,".bin")))
	        done=1;
	    else a++;
	} while(!done && a<cnt);
	if(a==cnt) RETFAIL("FAIL: MMU9: Zip file has no GBA binary.");

        MMUMainROMsize=ze.unc_size; strcpy(ARM9name,ze.name);
        sprintf(strout,"MMU9: Zipped ROM: %s, %d bytes.",ze.name,MMUMainROMsize);
        logvt->append(strout);
        GameROM=(RAMWORD*)malloc((MMUMainROMsize/4)*sizeof(RAMWORD));
        tempROM=(u8*)malloc(MMUMainROMsize);
        if(!GameROM || !tempROM) RETFAIL("FAIL: MMU9: GameROM allocation.");
        #ifdef MMUMainDEBUG
        logvt->append("MMU9: GameROM allocated.");
        #endif

        a=UnzipItemToBlock(hz,a,tempROM,MMUMainROMsize); if(a && a!=ZR_MORE)
        {
	    FormatZipMessage(a,str,256);
	    sprintf(strout,"FAIL: MMU9: Zip read: %s.",str);
	    RETFAIL(strout);
	}

        CloseZip(hz);
        #ifdef MMUMainDEBUG
        logvt->append("MMU9: Zip read and closed.");
        #endif
    } else {
        strcpy(ARM9name,MMUMainROMfile);
        logvt->append("MMU9: Opening ROM file.");
        hFile=CreateFile(MMUMainROMfile,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
        if(hFile==INVALID_HANDLE_VALUE) {
            FormatMessage(
                          FORMAT_MESSAGE_FROM_SYSTEM |
                          FORMAT_MESSAGE_IGNORE_INSERTS,
                          NULL, GetLastError(), 0, (LPTSTR)str, 256, NULL);
            sprintf(strout,"FAIL: MMU9: ROM file open: %s, %s.",str,MMUMainROMfile);
            RETFAIL(strout);
        }
        //#ifdef MMUMainDEBUG
        logvt->append("MMU9: ROM file open.");
        //#endif

        MMUMainROMsize=GetFileSize(hFile,NULL);
        sprintf(str, "MMU9: ROM size: %d bytes.",MMUMainROMsize);
        logvt->append(str);
        GameROM=(RAMWORD*)malloc(((MMUMainROMsize/4)+1)*sizeof(RAMWORD));
        tempROM=(u8*)malloc(MMUMainROMsize);
        if(!GameROM || !tempROM) RETFAIL("FAIL: MMU9: GameROM allocation.");
        //#ifdef MMUMainDEBUG
        logvt->append("MMU9: GameROM allocated.");
        //#endif

        ReadFile(hFile,tempROM,MMUMainROMsize,&bytesread,NULL);
        if(bytesread != MMUMainROMsize) RETFAIL("FAIL: ROM read.");
        CloseHandle(hFile); hFile=NULL;
        //#ifdef MMUMainDEBUG
        logvt->append("MMU9: ROM read and closed.");
        //#endif
    }

    for(a=0;a<MMUMainROMsize;a++)
    {
        #if WORDS_BIGENDIAN
        GameROM[a>>2].b[3-(a&3)]=tempROM[a];
        #else
        GameROM[a>>2].b[a&3]=tempROM[a];
        #endif
    }
    free(tempROM);
    for(a=0;a<(MMUMainROMsize>>2);a++)
    {
        GameROM[a]=ARM9opDecode(GameROM[a].data);
    }

    ARM9size=MMUMainROMsize;
    if(!strncmp((ARM9name+strlen(ARM9name)-3),"nds",3))
    {
        ARM9source+=GameROM[8].data;
        ARM9start=GameROM[9].data;
        ARM9dest=GameROM[10].data;
        ARM9size=GameROM[11].data;
    }
    sprintf(str,"MMU9: NDS: size=%08X, source=%08X, copyto=%08X, execfrom=%08X",
            ARM9size,ARM9source,ARM9dest,ARM9start);
    logvt->append(str);
    logvt->append("MMU9: About to copy ARM9 rom.");
    for(a=0;a<ARM9size;a+=4)
        MMUMainwrW(0,ARM9dest+a,MMUMainrdW(0,ARM9source+a));
    arm9reg.r[15]=ARM9start;

    hFile=CreateFile(BIOSpath,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
    if(!hFile) RETFAIL("FAIL: MMU9: BIOS file open.");
    MMUMainBIOSsize=GetFileSize(hFile,NULL);
    BIOS9=(RAMWORD*)malloc(MMUMainBIOSsize/4*sizeof(RAMWORD));
    tempROM=(u8*)malloc(MMUMainBIOSsize);
    if(!BIOS9 || !tempROM) RETFAIL("FAIL: MMU9: BIOS allocation.");
    ReadFile(hFile,tempROM,MMUMainBIOSsize,&bytesread,NULL);
    if(bytesread != MMUMainBIOSsize)
    {
        sprintf(str,"FAIL: BIOS read after %d bytes.",bytesread);
        RETFAIL(str);
    }
    CloseHandle(hFile); hFile=NULL;
    #ifdef MMUMainDEBUG
    logvt->append("MMU9: BIOS read and closed.");
    #endif

    for(a=0;a<MMUMainBIOSsize;a++)
    {
        #if WORDS_BIGENDIAN
        BIOS9[a>>2].b[3-(a&3)]=tempROM[a];
        #else
        BIOS9[a>>2].b[a&3]=tempROM[a];
        #endif
    }
    free(tempROM);
    for(a=0;a<MMUMainBIOSsize/4;a++)
    {
        BIOS9[a]=ARM9opDecode(BIOS9[a].data);
    }

    for(a=0;a<553;a++) VRAMmap[a]=(a&63);

    RETPASS("MMU9: Reset.");
}

void MMUMainfini()
{
    free(EWRAM);
    free(IWRAM);
    free(GameROM);
    logvt->append("MMU9: Shutdown.");
}

void MMUMainremapVRAM(u8 cntindex, u8 vcnt)
{
    u8 master=vcnt&7, offset=(vcnt>>3)&15, a;
    char str[160];
    if(!(vcnt&0x80)) return;
    sprintf(str,"VRAM_CNT_%d: master=%d, offset=%d",cntindex,master,offset);
    switch(cntindex)
    {
        case 0:
        sprintf(str,"%s: phys=00000",str);
        switch(master)  //->00000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=800000",str);
                    for(a=0;a<8;a++) VRAMmap[a+512]=a;
                } break;  //800000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a; break; //060000
                default: break;
            } break;
            case 2: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=400000",str); for(a=0;a<8;a++) VRAMmap[a+256]=a; break; //400000
                case 1: sprintf(str,"%s, virt=420000",str); for(a=0;a<8;a++) VRAMmap[a+264]=a; break; //420000
                default: break;
            } break;
            default: break;
        } break;
        case 1:
        sprintf(str,"%s: phys=20000",str);
        switch(master)  //->20000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=820000",str);
                    for(a=0;a<8;a++) VRAMmap[a+520]=a+8;
                } break;  //820000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+8; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+8; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+8; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+8; break; //060000
                default: break;
            } break;
            case 2: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=400000",str); for(a=0;a<8;a++) VRAMmap[a+256]=a+8; break; //400000
                case 1: sprintf(str,"%s, virt=420000",str); for(a=0;a<8;a++) VRAMmap[a+264]=a+8; break; //420000
                default: break;
            } break;
            default: break;
        } break;
        case 2:
        sprintf(str,"%s: phys=40000",str);
        switch(master)  //->40000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=840000",str);
                    for(a=0;a<8;a++) VRAMmap[a+528]=a+16;
                } break;  //840000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+16; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+16; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+16; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+16; break; //060000
                default: break;
            } break;
            case 4:
                if(!offset) {
                    sprintf(str,"%s, virt=200000",str);
                    for(a=0;a<8;a++) VRAMmap[a+128]=a+16;
                } break;  //200000
            default: break;
        } break;
        case 3:
        sprintf(str,"%s: phys=60000",str);
        switch(master)  //->60000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=860000",str);
                    for(a=0;a<8;a++) VRAMmap[a+536]=a+24;
                } break;  //860000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+24; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+24; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+24; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+24; break; //060000
                default: break;
            } break;
            case 4:
                if(!offset) {
                    sprintf(str,"%s, virt=600000",str);
                    for(a=0;a<8;a++) VRAMmap[a+384]=a+24;
                } break;  //600000
            default: break;
        } break;
        default: break;
    }
    logvt->append(str);
}

#define MMUMainintRdB(ram,addr) b=ram[(addr)>>2].b[(addr)&3]
#define MMUMainintRdH(ram,addr) \
    h=((addr)&1) \
        ?(((addr)&2) \
          ?(ram[(addr)>>2].b[3] | ((ram[((addr)>>2)+1].b[0])<<8)) \
          :(ram[(addr)>>2].b[1] | ((ram[(addr)>>2].b[2])<<8))) \
        :(ram[(addr)>>2].h[((addr)&2)>>1])
#define MMUMainintRdW(ram,addr) \
    switch((addr)&3) \
    { \
        case 0: w=ram[(addr)>>2].data; break; \
        case 1: w=(ram[(addr)>>2].data>> 8)+((ram[(addr)>>2].data&0x000000FF)<<24); break; \
        case 2: w=ram[(addr)>>2].h[1]+((ram[(addr)>>2].h[0])<<16); break; \
        case 3: w=(ram[(addr)>>2].data>>24)+((ram[(addr)>>2].data&0x00FFFFFF)<< 8); break; \
    }

u8 MMUMainrdB(u32 prot, u32 addr)
{
    u8 b; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) {
                b=DTCM[(addr&0x3FFF)>>2].b[addr&3];
            } else {
                b=ITCM[(addr&0x3FFF)>>2].b[addr&3];
            }
            break;
        case 0x02000000:
	    MMUMainintRdB(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU9: EWRAMRd: %08X=%02X",addr,b);
//                logvt->append(str);
            }
	    ARM9addClock(2); break;
	case 0x03000000:
	    MMUMainintRdB(IWRAM, addr&0x00007FFF);
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM9_REG,ARM9_REG,addr).flags&REG_FLAG_R)
                b=REG_ADDR(ARM9_REG,addr).b[addr&1];
            else b=0;
            if((!prot) /*&& mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU9: IORd %08X: %03X=%02X",arm9reg.r[0xf],addr&0x1FFF,b);
				logvt->append(str);
            }
	    break;
	case 0x05000000:
            b=GPUPAL8[addr&0x7FF];
            break;
	case 0x06000000:
	    b=(addr&1)?VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]>>8
                      :VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]&255;
	    break;
	case 0x07000000:
	    b=(addr&1)?OAM[(addr&0x7FF)>>1]>>8:OAM[(addr&0x7FF)>>1]&255;
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: OAMRd: %03X=%02X",addr&0x7FF,b);
            logvt->append(str);
            #endif
	    break;
	case 0x08000000: case 0x09000000:
	    if((addr&0x00FFFFFF)<=MMUMainROMsize)
                MMUMainintRdB(GameROM, addr&0x00FFFFFF);
            else b=MMUMainEmpty.b[addr&3];
	    ARM9addClock(4); break;
        case 0x0F000000:
            if(((addr&0xFFFF0000)==0xFFFF0000) && prot)
                MMUMainintRdB(BIOS9, addr&0x000001FF);
            else b=0;
            break;
	default: break;
    }
    return b;
}

u16 MMUMainrdH(u32 prot, u32 addr)
{
    u16 h; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) {
                h=DTCM[(addr&0x3FFF)>>2].h[(addr&2)>>1];
            } else {
                h=ITCM[(addr&0x3FFF)>>2].h[(addr&2)>>1];
            }
            break;
        case 0x02000000:
	    MMUMainintRdH(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU9: EWRAMRd: %08X=%04X",addr,h);
//                logvt->append(str);
            }
	    ARM9addClock(2); break;
	case 0x03000000:
	    MMUMainintRdH(IWRAM, addr&0x00007FFF);
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_R)
                h=REG_ADDR(ARM9_REG,addr).data;
            else h=0;
            if((!prot)/* && mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU9: IORd %08X: %03X=%04X",arm9reg.r[0xf],addr&0x1FFF,h);
				logvt->append(str);
            }
	    break;
	case 0x05000000:
	    h=GPUPAL[(addr>>1)&0x3FF];
            break;
	case 0x06000000:
	    h=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1];
	    break;
	case 0x07000000:
	    h=OAM[(addr&0x7FF)>>1];
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: OAMRd: %03X=%04X",addr&0x7FF,h);
            logvt->append(str);
            #endif
	    break;
	case 0x08000000: case 0x09000000:
	    if((addr&0x00FFFFFF)<=MMUMainROMsize)
                MMUMainintRdH(GameROM, addr&0x00FFFFFF);
            else h=MMUMainEmpty.h[(addr&2)>>1];
	    ARM9addClock(4); break;
        case 0x0F000000:
            if(prot) MMUMainintRdH(BIOS9, addr&0x000001FF); else return 0;
            break;
	default: break;
    }
    return h;
}

u32 MMUMainrdW(u32 prot, u32 addr)
{
    u32 w; u16 hl,hh; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) {
                w=DTCM[(addr&0x3FFF)>>2].data;
            } else {
                w=ITCM[(addr&0x3FFF)>>2].data;
            }
            break;
        case 0x02000000:
	    MMUMainintRdW(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU9: EWRAMRd: %08X=%08X",addr,w);
//                logvt->append(str);
            }
	    ARM9addClock(5); break;
	case 0x03000000:
	    MMUMainintRdW(IWRAM, addr&0x00007FFF);
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_R)
                hl=REG_ADDR(ARM9_REG,addr).data;
            else hl=0;
	    addr+=2;
	    if(prot || REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_R)
                hh=REG_ADDR(ARM9_REG,addr).data;
            else hh=0;
            w=hl+(hh<<16);
            if((!prot)/* && mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU9: IORd %08X: %03X=%08X",arm9reg.r[0xf],(addr-2)&0x1FFF,w);
				logvt->append(str);
            }
	    break;
	case 0x05000000:
	    hl=GPUPAL[(addr>>1)&0x3FF];
            addr+=2;
	    hh=GPUPAL[(addr>>1)&0x3FF];
            w=hl+(hh<<16);
            break;
	case 0x06000000:
	    hl=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]; addr+=2;
	    hh=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1];
            w=hl+(hh<<16);
            break;
	case 0x07000000:
	    hl=OAM[(addr&0x7FF)>>1]; addr+=2;
	    hh=OAM[(addr&0x7FF)>>1];
            w=hl+(hh<<16);
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: OAMRd: %03X=%08X",addr&0x7FF,w);
            logvt->append(str);
            #endif
            break;
	case 0x08000000: case 0x09000000:
//            sprintf(str,"MMU9: RdW at %08X",addr); logvt->append(str);
	    if((addr&0x00FFFFFF)<=MMUMainROMsize){ MMUMainintRdW(GameROM, addr&0x00FFFFFF); }
            else w=MMUMainEmpty.data;
	    ARM9addClock(7); break;
        case 0x0F000000:
            if(prot) {MMUMainintRdW(BIOS9, addr&0x000001FF);} else return 0;
            break;
	default: break;
    }
    return w;
}

RAMWORD MMUMainrdS(u32 prot, u32 addr)
{
    if((addr&0xFE000000)==0x08000000)
    {
        if((addr&0x00FFFFFF)<=MMUMainROMsize) return GameROM[(addr&0xFFFFFF)>>2];
        else return MMUMainEmpty;
    }
    switch(addr&0x0F000000)
    {
        case 0x03000000: return IWRAM[(addr&0x07FFF)>>2];
        case 0x02000000: return EWRAM[(addr&0x3FFFFF)>>2];
        case 0x00000000: if(!(addr&0x00800000)) 
							 return ITCM[(addr&0x0FFF)>>2];
						 else
							 return DTCM[(addr&0x3fff)>>2];
        case 0x0F000000: if((addr&0xFFFF0000)==0xFFFF0000) {
							 return BIOS9[(addr&0x001FF)>>2];
						 }
    }
    return MMUMainEmpty;
}

#define MMUMainintWrB(ram,addr,val) \
    ram[(addr)>>2].b[(addr)&3]=val; \
    ram[(addr)>>2].op=ARM9opUNL; \
    ram[(addr)>>2].cond=ARM9condAL

#define MMUMainintWrH(ram,addr,val) \
    ram[(addr)>>2].h[((addr)&2)>>1]=val; \
    ram[(addr)>>2].op=ARM9opUNL; \
    ram[(addr)>>2].cond=ARM9condAL

#define MMUMainintWrW(ram,addr,val) \
    ram[(addr)>>2].data=val; \
    ram[(addr)>>2].op=ARM9opUNL; \
    ram[(addr)>>2].cond=ARM9condAL

void MMUMainwrB(u32 bus, u32 addr, u8 val)
{
    char str[80];
#if 0
    sprintf(str,"MMU9: MMUMainwrB %08X: %08X=%02X",arm9reg.r[0xf],addr,val);
                    logvt->append(str);
#endif

    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].b[addr&3]=val;
            else MMUMainintWrB(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUMainintWrB(EWRAM, addr&0x0003FFFFF, val);
	    ARM9addClock(2); break;
	case 0x03000000:
	    MMUMainintWrB(IWRAM, addr&0x00007FFF, val);
	    break;
        case 0x04000000:
            if(REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) 
					IntClear(val);
                else REG_ADDR(ARM9_REG,addr).b[addr&1]=val;
                if((addr&0x1FF0)==0x240) MMUMainremapVRAM(addr&15,val);
                DMAcheck(DMA_TIMENOW);
                /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
                {
                    sprintf(str,"MMU9: IOWr %08X: %03X=%02X",arm9reg.r[0xf],addr&0x1FFF,val);
                    logvt->append(str);
                }
            }
	default: break;
    }
}

void MMUMainwrH(u32 bus, u32 addr, u16 val)
{
    char str[80];
#if 0
sprintf(str,"MMU9: MMUMainwrH %08X: %08X=%04X",arm9reg.r[0xf],addr,val);
                    logvt->append(str);
#endif
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].h[(addr&2)>>1]=val;
            else MMUMainintWrH(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUMainintWrH(EWRAM, addr&0x0003FFFFF, val);
	    ARM9addClock(2); break;
	case 0x03000000:
	    MMUMainintWrH(IWRAM, addr&0x00007FFF, val);
	    break;
	case 0x04000000:
            if(REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val);
                else REG_ADDR(ARM9_REG,addr).data=val;
                if((addr&0x1FF0)==0x240)
                {
                    MMUMainremapVRAM((addr+0)&15,(val>> 0)&255);
                    MMUMainremapVRAM((addr+1)&15,(val>> 8)&255);
                }
                DMAcheck(DMA_TIMENOW);
                /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
                {
                    sprintf(str,"MMU9: IOWr %08X: %03X=%04X",arm9reg.r[0xf],addr&0x1FFF,val);
                    logvt->append(str);
                }
            }
	    break;
	case 0x05000000:
	    #ifdef MMUMainDEBUG
	    sprintf(str,"MMU9: PalWr: %08X=%04X",addr,val);
	    logvt->append(str);
	    #endif
	    GPUPAL[(addr>>1)&0x3FF]=val;
	    break;
	case 0x06000000:
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: VRAMWr: %08X [%06X=%06X]=%04X",addr,((addr>>14)&1023)<<14,0x800000+(VRAMmap[(addr>>14)&1023]<<14),val);
            logvt->append(str);
            #endif
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val; break;
	case 0x07000000:
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: OAMWr: %03X=%04X",addr&0x7FF,val);
            logvt->append(str);
            #endif
	    OAM[(addr&0x7FF)>>1]=val;
            break;
	default: break;
    }
}

void MMUMainwrW(u32 bus, u32 addr, u32 val)
{
    char str[80];
#if 0
sprintf(str,"MMU9: MMUMainwrW %08X: %08X=%08X",arm9reg.r[0xf],addr,val);
                    logvt->append(str);
#endif

    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].data=val;
            else MMUMainintWrW(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUMainintWrW(EWRAM, addr&0x0003FFFFF, val);
	    ARM9addClock(2); break;
	case 0x03000000:
	    MMUMainintWrW(IWRAM, addr&0x00007FFF, val);
	    break;
	case 0x04000000:
            if(REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val&65535);
                else REG_ADDR(ARM9_REG,addr).data=val&65535;
                if((addr&0x1FF0)==0x240)
                {
                    MMUMainremapVRAM((addr+0)&15,(val>> 0)&255);
                    MMUMainremapVRAM((addr+1)&15,(val>> 8)&255);
                }
            }
            /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
            {
                sprintf(str,"MMU9: IOWr %08X: %03X=%08X",arm9reg.r[0xf],addr&0x1FFF,val);
                logvt->append(str);
            }
	    addr+=2;
            if(REG_ADDR(ARM9_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val>>16);
                else REG_ADDR(ARM9_REG,addr).data=val>>16;
                if((addr&0x1FF0)==0x240)
                {
                    MMUMainremapVRAM((addr+0)&15,(val>>16)&255);
                    MMUMainremapVRAM((addr+1)&15,(val>>24)&255);
                }
            }
            DMAcheck(DMA_TIMENOW);
	    break;
	case 0x05000000:
	    sprintf(str,"MMU9: PalWr: %08X=%08X",addr,val);
	    logvt->append(str);
	    GPUPAL[(addr>>1)&0x3FF]=val&65535;
	    addr+=2;
	    GPUPAL[(addr>>1)&0x3FF]=val>>16;
	case 0x06000000:
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: VRAMWr: %08X [%06X=%06X]=%08X",addr,((addr>>14)&1023)<<14,0x800000+(VRAMmap[(addr>>14)&1023]<<14),val);
            logvt->append(str);
            #endif
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val&65535; addr+=2;
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val>>16;
	    break;
	case 0x07000000:
	    #ifdef MMUMainDEBUG
            sprintf(str,"MMU9: OAMWr: %03X=%08X",addr&0x7FF,val);
            logvt->append(str);
            #endif
	    OAM[(addr&0x7FF)>>1]=val&65535; addr+=2;
	    OAM[(addr&0x7FF)>>1]=val>>16;
	    break;
	default: break;
    }
}

void MMUMainwrS(u32 bus, u32 addr, RAMWORD val)
{
	switch(addr&0x0F000000)
    {
        case 0x00000000: if(!(addr&0x00800000)) ITCM[(addr&0x0FFF)>>2]=val; break;
        case 0x02000000: EWRAM[(addr&0x3FFFFF)>>2]=val; break;
        case 0x03000000: IWRAM[(addr&0x07FFF)>>2]=val; break;
	case 0x08000000:
	case 0x09000000:
	case 0x0A000000:
	case 0x0B000000:
	case 0x0C000000:
        case 0x0D000000: GameROM[(addr&0xFFFFFF)>>2]=val; break;
	default: break;
    }
}
