/**************************************************************************
* DSemu: Memory management and handling (mmu.c)                           *
* Released under the terms of the BSD Public Licence                      *
* Imran Nazar (tf@oopsilon.com), 2004                                     *
**************************************************************************/

#include <stdio.h>
#include "vtbl.h"
#include "err.h"
#include "arm.h"
#include "dsmmusub.h"
#include "dma.h"
#include "dsioreg.h"
#include "gpu.h"
#include "unzip.h"

//#define MMUDEBUG
//#define BIOSsize 1024

extern u16 *VRAM;
extern u8  *VRAM8;
extern RAMWORD *EWRAM, *IWRAM, *GameROM;
RAMWORD *BIOS;

// The ARM7 has access to 64K of RAM exclusive to its own use.
RAMWORD* ARM7_IWRAM;	

extern ARMREGS arm7reg;

extern u16 OAM[512];
extern u32 LCDcolour[32768];
extern u16 GPUPAL[1024];
extern u8 *GPUPAL8;

extern ARMREGS arm7reg;

RAMWORD MMUSubEmpty;
u32 MMUSubROMsize; char *MMUSubROMfile;
u32 MMUSubBIOSsize;
u32 VRAMmap[553];

RAMWORD DTCM[4096], ITCM[4096];

int MMUinit(char *file)
{
    FILE *fp; u8 c; int a;

    MMUSubROMfile=file;

    MMUSubEmpty.data=0xFFFFFFFF;
    MMUSubEmpty.op=ARM7opUNL;
    MMUSubEmpty.cond=ARM7condAL;

    ARM7_IWRAM=(RAMWORD*)malloc(16384*sizeof(RAMWORD));
    if(!ARM7_IWRAM) RETFAIL("FAIL: MMU7: ARM7_IWRAM allocation.");


    RETPASS("MMU7: Memoryspace allocation complete.");
}

int MMUreset()
{
    HANDLE hFile; u8 c; int a,done=0,bytesread; u8 *tempROM;
    char BIOSpath[1024], INIpath[1024], ARM7name[1024], str[256], strout[384];
    int pathsize;
    HZIP hz; ZIPENTRY ze; int cnt;
    u32 ARM7size, ARM7source=0x08000000, ARM7dest=0x02380000, ARM7start=0x02380000;

	// Reset the ARM7's own 64K RAM area to 0.
    memset(ARM7_IWRAM,0,0xFFFF);

    pathsize=strstr(logvt->file,"\\log.txt")-logvt->file;
    strncpy(BIOSpath, logvt->file, pathsize); BIOSpath[pathsize]=0;
    sprintf(INIpath, "%s\\dsemu.ini", BIOSpath);
    GetPrivateProfileString("General","BIOS","bioshack.bin",str,256,INIpath);
    sprintf(BIOSpath, "%s\\%s", BIOSpath,str);

    if(!strncmp((MMUSubROMfile+strlen(MMUSubROMfile)-3),"zip",3))
    {
        hz=OpenZipFromName(MMUSubROMfile,0);
        if(!hz) RETFAIL("FAIL: MMU7: Zip file open.");
        #ifdef MMUDEBUG
        logvt->append("MMU7: Zip file open.");
        #endif
        GetZipItem(hz,-1,&ze); cnt=ze.index; a=0;
        do {
            GetZipItem(hz,a,&ze);
            if(ze.unc_size && (strstr(ze.name,".gba") || strstr(ze.name,".bin")))
	        done=1;
	    else a++;
	} while(!done && a<cnt);
	if(a==cnt) RETFAIL("FAIL: MMU7: Zip file has no GBA binary.");

        MMUSubROMsize=ze.unc_size; strcpy(ARM7name,ze.name);
        sprintf(strout,"MMU7: Zipped ROM: %s, %d bytes.",ze.name,MMUSubROMsize);
        logvt->append(strout);
        CloseZip(hz);
//        #ifdef MMUDEBUG
        logvt->append("MMU7: Zip not read, closed.");
//        #endif
    } else {
        strcpy(ARM7name,MMUSubROMfile);
        hFile=CreateFile(MMUSubROMfile,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
        if(hFile==INVALID_HANDLE_VALUE) {
            FormatMessage(
                          FORMAT_MESSAGE_FROM_SYSTEM |
                          FORMAT_MESSAGE_IGNORE_INSERTS,
                          NULL, GetLastError(), 0, (LPTSTR)str, 256, NULL);
            sprintf(strout,"FAIL: MMU7: ROM file open: %s,%s",str,MMUSubROMfile);
            RETFAIL(strout);
        }
//        #ifdef MMUDEBUG
        logvt->append("MMU7: ROM file open.");
//       #endif

        MMUSubROMsize=GetFileSize(hFile,NULL);
        sprintf(str, "MMU7: ROM size: %d bytes.",MMUSubROMsize);
        logvt->append(str);
        CloseHandle(hFile); hFile=NULL;
//        #ifdef MMUDEBUG
        logvt->append("MMU7: ROM NOT read, closed.");
//        #endif
    }

    ARM7size=MMUSubROMsize;
    if(!strncmp((ARM7name+strlen(ARM7name)-3),"nds",3))
    {
        ARM7source+=GameROM[0x0c].data;
        ARM7start=GameROM[0x0d].data;
        ARM7dest=GameROM[0x0e].data;
        ARM7size=GameROM[0x0f].data;
        sprintf(str,"MMU7: NDS: size=%08X, source=%08X, copyto=%08X, execfrom=%08X",
                ARM7size,ARM7source,ARM7dest,ARM7start);
        logvt->append(str);
        logvt->append("MMU7: About to copy ARM7 rom.");
        for(a=0;a<ARM7size;a+=4)
            MMUwrW(0,ARM7dest+a,MMUrdW(0,ARM7source+a));
    } else {
        MMUwrW(0,ARM7start,0xE2900001);
        MMUwrW(0,ARM7start+4,0xEAFFFFFD);
    }
    arm7reg.r[15]=ARM7start;

	/* Load ARM7 Bios from file */
    hFile=CreateFile(BIOSpath,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
    if(!hFile) RETFAIL("FAIL: MMU7: BIOS file open.");
    MMUSubBIOSsize=GetFileSize(hFile,NULL);
    BIOS=(RAMWORD*)malloc(MMUSubBIOSsize/4*sizeof(RAMWORD));
    tempROM=(u8*)malloc(MMUSubBIOSsize);
    if(!BIOS || !tempROM) RETFAIL("FAIL: MMU7: BIOS allocation.");
    ReadFile(hFile,tempROM,MMUSubBIOSsize,&bytesread,NULL);
    if(bytesread != MMUSubBIOSsize)
    {
        sprintf(str,"FAIL: BIOS read after %d bytes.",bytesread);
        RETFAIL(str);
    }
    CloseHandle(hFile); hFile=NULL;
    #ifdef MMUDEBUG
    logvt->append("MMU7: BIOS read and closed.");
    #endif

    for(a=0;a<MMUSubBIOSsize;a++)
    {
        #if WORDS_BIGENDIAN
        BIOS[a>>2].b[3-(a&3)]=tempROM[a];
        #else
        BIOS[a>>2].b[a&3]=tempROM[a];
        #endif
    }
    free(tempROM);
    for(a=0;a<MMUSubBIOSsize/4;a++)
    {
        BIOS[a]=ARM7opDecode(BIOS[a].data);
    }

    for(a=0;a<553;a++) VRAMmap[a]=(a&63);

    RETPASS("MMU7: Reset.");
}

void MMUfini()
{
	free(ARM7_IWRAM);
    logvt->append("MMU7: Shutdown.");
}

void MMUremapVRAM(u8 cntindex, u8 vcnt)
{
    u8 master=vcnt&7, offset=(vcnt>>3)&15, a;
    char str[160];
    if(!(vcnt&0x80)) return;
    sprintf(str,"VRAM_CNT_%d: master=%d, offset=%d",cntindex,master,offset);
    switch(cntindex)
    {
        case 0:
        sprintf(str,"%s: phys=00000",str);
        switch(master)  //->00000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=800000",str);
                    for(a=0;a<8;a++) VRAMmap[a+512]=a;
                } break;  //800000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a; break; //060000
                default: break;
            } break;
            case 2: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=400000",str); for(a=0;a<8;a++) VRAMmap[a+256]=a; break; //400000
                case 1: sprintf(str,"%s, virt=420000",str); for(a=0;a<8;a++) VRAMmap[a+264]=a; break; //420000
                default: break;
            } break;
            default: break;
        } break;
        case 1:
        sprintf(str,"%s: phys=20000",str);
        switch(master)  //->20000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=820000",str);
                    for(a=0;a<8;a++) VRAMmap[a+520]=a+8;
                } break;  //820000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+8; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+8; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+8; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+8; break; //060000
                default: break;
            } break;
            case 2: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=400000",str); for(a=0;a<8;a++) VRAMmap[a+256]=a+8; break; //400000
                case 1: sprintf(str,"%s, virt=420000",str); for(a=0;a<8;a++) VRAMmap[a+264]=a+8; break; //420000
                default: break;
            } break;
            default: break;
        } break;
        case 2:
        sprintf(str,"%s: phys=40000",str);
        switch(master)  //->40000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=840000",str);
                    for(a=0;a<8;a++) VRAMmap[a+528]=a+16;
                } break;  //840000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+16; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+16; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+16; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+16; break; //060000
                default: break;
            } break;
            case 4:
                if(!offset) {
                    sprintf(str,"%s, virt=200000",str);
                    for(a=0;a<8;a++) VRAMmap[a+128]=a+16;
                } break;  //200000
            default: break;
        } break;
        case 3:
        sprintf(str,"%s: phys=60000",str);
        switch(master)  //->60000
        {
            case 0:
                if(!offset) {
                    sprintf(str,"%s, virt=860000",str);
                    for(a=0;a<8;a++) VRAMmap[a+536]=a+24;
                } break;  //860000
            case 1: switch(offset)
            {
                case 0: sprintf(str,"%s, virt=000000",str); for(a=0;a<8;a++) VRAMmap[a   ]=a+24; break; //000000
                case 1: sprintf(str,"%s, virt=020000",str); for(a=0;a<8;a++) VRAMmap[a+8 ]=a+24; break; //020000
                case 2: sprintf(str,"%s, virt=040000",str); for(a=0;a<8;a++) VRAMmap[a+16]=a+24; break; //040000
                case 3: sprintf(str,"%s, virt=060000",str); for(a=0;a<8;a++) VRAMmap[a+24]=a+24; break; //060000
                default: break;
            } break;
            case 4:
                if(!offset) {
                    sprintf(str,"%s, virt=600000",str);
                    for(a=0;a<8;a++) VRAMmap[a+384]=a+24;
                } break;  //600000
            default: break;
        } break;
        default: break;
    }
    logvt->append(str);
}

#define MMUintRdB(ram,addr) b=ram[(addr)>>2].b[(addr)&3]
#define MMUintRdH(ram,addr) \
    h=((addr)&1) \
        ?(((addr)&2) \
          ?(ram[(addr)>>2].b[3] | ((ram[((addr)>>2)+1].b[0])<<8)) \
          :(ram[(addr)>>2].b[1] | ((ram[(addr)>>2].b[2])<<8))) \
        :(ram[(addr)>>2].h[((addr)&2)>>1])
#define MMUintRdW(ram,addr) \
    switch((addr)&3) \
    { \
        case 0: w=ram[(addr)>>2].data; break; \
        case 1: w=(ram[(addr)>>2].data>> 8)+((ram[(addr)>>2].data&0x000000FF)<<24); break; \
        case 2: w=ram[(addr)>>2].h[1]+((ram[(addr)>>2].h[0])<<16); break; \
        case 3: w=(ram[(addr)>>2].data>>24)+((ram[(addr)>>2].data&0x00FFFFFF)<< 8); break; \
    }

u8 MMUrdB(u32 prot, u32 addr)
{
    u8 b; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            MMUintRdB(BIOS, addr&0x00003FFF);
            break;
        case 0x02000000:
	    MMUintRdB(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU7: EWRAMRd: %08X=%02X",addr,b);
//                logvt->append(str);
            }
	    ARM7addClock(2); break;
	case 0x03000000:
		if(addr&0x00800000) { 
		    MMUintRdB(ARM7_IWRAM, addr&0x0000FFFF);
		}
		else {
			MMUintRdB(IWRAM, addr&0x00007FFF);
		}
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_R)
                b=REG_ADDR(ARM7_REG,addr).b[addr&1];
            else b=0;
            #ifdef MMUDEBUG
            if((!prot) /*&& mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU7: IORd: %03X=%02X",addr&0x1FFF,b);
//		logvt->append(str);
            }
            #endif
	    break;
	case 0x05000000:
            b=GPUPAL8[addr&0x7FF];
            break;
	case 0x06000000:
	    b=(addr&1)?VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]>>8
                      :VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]&255;
	    break;
	case 0x07000000:
	    b=(addr&1)?OAM[(addr&0x7FF)>>1]>>8:OAM[(addr&0x7FF)>>1]&255;
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: OAMRd: %03X=%02X",addr&0x7FF,b);
            logvt->append(str);
            #endif
	    break;
	case 0x08000000: case 0x09000000:
	    if((addr&0x00FFFFFF)<=MMUSubROMsize)
                MMUintRdB(GameROM, addr&0x00FFFFFF);
            else b=MMUSubEmpty.b[addr&3];
	    ARM7addClock(4); break;
	default: break;
    }
    return b;
}

u16 MMUrdH(u32 prot, u32 addr)
{
    u16 h; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            MMUintRdH(BIOS, addr&0x00003FFF);
            break;
        case 0x02000000:
	    MMUintRdH(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU7: EWRAMRd: %08X=%04X",addr,h);
//                logvt->append(str);
            }
	    ARM7addClock(2); break;
	case 0x03000000:
		if(addr&0x00800000) {
		    MMUintRdH(ARM7_IWRAM, addr&0x0000FFFF);
		}
		else {
		    MMUintRdH(IWRAM, addr&0x00007FFF);
		}
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_R)
                h=REG_ADDR(ARM7_REG,addr).data;
            else h=0;
            #ifdef MMUDEBUG
            if((!prot)/* && mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU7: IORd: %03X=%04X",addr&0x1FFF,h);
//		logvt->append(str);
            }
            #endif
	    break;
	case 0x05000000:
	    h=GPUPAL[(addr>>1)&0x3FF];
            break;
	case 0x06000000:
	    h=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1];
	    break;
	case 0x07000000:
	    h=OAM[(addr&0x7FF)>>1];
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: OAMRd: %03X=%04X",addr&0x7FF,h);
            logvt->append(str);
            #endif
	    break;
	case 0x08000000: case 0x09000000:
	    if((addr&0x00FFFFFF)<=MMUSubROMsize)
                MMUintRdH(GameROM, addr&0x00FFFFFF);
            else h=MMUSubEmpty.h[(addr&2)>>1];
	    ARM7addClock(4); break;
	default: break;
    }
    return h;
}

u32 MMUrdW(u32 prot, u32 addr)
{
    u32 w; u16 hl,hh; char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            MMUintRdW(BIOS, addr&0x00003FFF);
            break;
        case 0x02000000:
	    MMUintRdW(EWRAM, addr&0x0003FFFFF);
	    if(((addr&0xFFFFFF00)==0x02000000) ||
               ((addr&0xFFFFFF00)==0x0237F000))
            {
//                sprintf(str,"MMU7: EWRAMRd: %08X=%08X",addr,w);
//                logvt->append(str);
            }
	    ARM7addClock(5); break;
	case 0x03000000:
		if(addr&0x00800000) {
			MMUintRdW(ARM7_IWRAM, addr&0x0000FFFF);
		}
		else  {
			MMUintRdW(IWRAM, addr&0x00007FFF);
		}
	    break;
	case 0x04000000:
	    if(prot || REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_R)
                hl=REG_ADDR(ARM7_REG,addr).data;
            else hl=0;
	    addr+=2;
	    if(prot || REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_R)
                hh=REG_ADDR(ARM7_REG,addr).data;
            else hh=0;
            w=hl+(hh<<16);
            #ifdef MMUDEBUG
            if((!prot)/* && mmuioimpl[(addr&0x1FFF)>>1]==1*/)
            {
                sprintf(str,"MMU7: IORd: %03X=%08X",addr&0x1FFF,w);
//		logvt->append(str);
            }
            #endif
	    break;
	case 0x05000000:
	    hl=GPUPAL[(addr>>1)&0x3FF];
            addr+=2;
	    hh=GPUPAL[(addr>>1)&0x3FF];
            w=hl+(hh<<16);
            break;
	case 0x06000000:
	    hl=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]; addr+=2;
	    hh=VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1];
            w=hl+(hh<<16);
            break;
	case 0x07000000:
	    hl=OAM[(addr&0x7FF)>>1]; addr+=2;
	    hh=OAM[(addr&0x7FF)>>1];
            w=hl+(hh<<16);
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: OAMRd: %03X=%08X",addr&0x7FF,w);
            logvt->append(str);
            #endif
            break;
	case 0x08000000: case 0x09000000:
//            sprintf(str,"MMU7: RdW at %08X",addr); logvt->append(str);
	    if((addr&0x00FFFFFF)<=MMUSubROMsize){ MMUintRdW(GameROM, addr&0x00FFFFFF); }
            else w=MMUSubEmpty.data;
	    ARM7addClock(7); break;
	default: break;
    }
    return w;
}

RAMWORD MMUrdS(u32 prot, u32 addr)
{
    if((addr&0xFE000000)==0x08000000)
    {
        if((addr&0x00FFFFFF)<=MMUSubROMsize) return GameROM[(addr&0xFFFFFF)>>2];
        else return MMUSubEmpty;
    }
    switch(addr&0x0F000000)
    {
        case 0x03000000: 
			if(addr&0x00800000) 
				return ARM7_IWRAM[(addr&0x0FFFF)>>2];
			else
				return IWRAM[(addr&0x07FFF)>>2];

        case 0x02000000: return EWRAM[(addr&0x3FFFFF)>>2];
        case 0x00000000: 
            return BIOS[(addr&0x1ff)>>2];
    }
    return MMUSubEmpty;
}
#define MMUintWrB(ram,addr,val) \
    ram[(addr)>>2].b[(addr)&3]=val; \
    ram[(addr)>>2].op=ARM7opUNL; \
    ram[(addr)>>2].cond=ARM7condAL

#define MMUintWrH(ram,addr,val) \
    ram[(addr)>>2].h[((addr)&2)>>1]=val; \
    ram[(addr)>>2].op=ARM7opUNL; \
    ram[(addr)>>2].cond=ARM7condAL

#define MMUintWrW(ram,addr,val) \
    ram[(addr)>>2].data=val; \
    ram[(addr)>>2].op=ARM7opUNL; \
    ram[(addr)>>2].cond=ARM7condAL

void MMUwrB(u32 bus, u32 addr, u8 val)
{
    char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].b[addr&3]=val;
            else MMUintWrB(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUintWrB(EWRAM, addr&0x0003FFFFF, val);
	    ARM7addClock(2); break;
	case 0x03000000:
		if(addr&0x0800000) {
			MMUintWrB(ARM7_IWRAM, addr&0x0000FFFF, val);
		}
		else {
			MMUintWrB(IWRAM, addr&0x00007FFF, val);
		}
	    break;
        case 0x04000000:
            if(REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) 
					IntClear(val);
                else REG_ADDR(ARM7_REG,addr).b[addr&1]=val;
                if((addr&0x1FF0)==0x240) MMUremapVRAM(addr&15,val);
                DMAcheck(DMA_TIMENOW);
            #ifdef MMUDEBUG
                /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
                {
                    sprintf(str,"MMU7: IOWr: %03X=%02X",addr&0x1FFF,val);
                    logvt->append(str);
                }
            #endif
            }
	default: break;
    }
}

void MMUwrH(u32 bus, u32 addr, u16 val)
{
    char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].h[(addr&2)>>1]=val;
            else MMUintWrH(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUintWrH(EWRAM, addr&0x0003FFFFF, val);
	    ARM7addClock(2); break;
	case 0x03000000:
		if(addr&0x0800000) {
			MMUintWrH(ARM7_IWRAM, addr&0x0000FFFF, val);
		}
		else {
			MMUintWrH(IWRAM, addr&0x00007FFF, val);
		}
	    break;
	case 0x04000000:
            if(REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val);
                else REG_ADDR(ARM7_REG,addr).data=val;
                if((addr&0x1FF0)==0x240)
                {
                    MMUremapVRAM((addr+0)&15,(val>> 0)&255);
                    MMUremapVRAM((addr+1)&15,(val>> 8)&255);
                }
                DMAcheck(DMA_TIMENOW);
            #ifdef MMUDEBUG
                /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
                {
                    sprintf(str,"MMU7: IOWr: %03X=%04X",addr&0x1FFF,val);
                    logvt->append(str);
                }
            #endif
            }
	    break;
	case 0x05000000:
	    #ifdef MMUDEBUG
	    sprintf(str,"MMU7: PalWr: %08X=%04X",addr,val);
	    logvt->append(str);
	    #endif
	    GPUPAL[(addr>>1)&0x3FF]=val;
	    break;
	case 0x06000000:
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: VRAMWr: %08X [%06X=%06X]=%04X",addr,((addr>>14)&1023)<<14,0x800000+(VRAMmap[(addr>>14)&1023]<<14),val);
            logvt->append(str);
            #endif
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val; break;
	case 0x07000000:
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: OAMWr: %03X=%04X",addr&0x7FF,val);
            logvt->append(str);
            #endif
	    OAM[(addr&0x7FF)>>1]=val;
            break;
	default: break;
    }
}

void MMUwrW(u32 bus, u32 addr, u32 val)
{
    char str[80];
    switch(addr&0x0F000000)
    {
        case 0x00000000:
            if(addr&0x00800000) DTCM[(addr&0x3FFF)>>2].data=val;
            else MMUintWrW(ITCM,addr&0x0FFF,val);
            break;
        case 0x02000000:
	    MMUintWrW(EWRAM, addr&0x0003FFFFF, val);
	    ARM7addClock(2); break;
	case 0x03000000:
		if(addr&0x0800000) {
			MMUintWrW(ARM7_IWRAM, addr&0x0000FFFF, val);
		}
		else {
			MMUintWrW(IWRAM, addr&0x00007FFF, val);
		}
	    break;
	case 0x04000000:
            if(REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val&65535);
                else REG_ADDR(ARM7_REG,addr).data=val&65535;
                if((addr&0x1FF0)==0x240)
                {
                    MMUremapVRAM((addr+0)&15,(val>> 0)&255);
                    MMUremapVRAM((addr+1)&15,(val>> 8)&255);
                }
            }
            #ifdef MMUDEBUG
            /*if(mmuioimpl[(addr&0x3FF)>>1]==1)*/
            {
                sprintf(str,"MMU7: IOWr: %03X=%08X",addr&0x1FFF,val);
                logvt->append(str);
            }
            #endif
	    addr+=2;
            if(REG_ADDR(ARM7_REG,addr).flags&REG_FLAG_W)
            {
                if((addr&0x1FFE)==0x214) IntClear(val>>16);
                else REG_ADDR(ARM7_REG,addr).data=val>>16;
                if((addr&0x1FF0)==0x240)
                {
                    MMUremapVRAM((addr+0)&15,(val>>16)&255);
                    MMUremapVRAM((addr+1)&15,(val>>24)&255);
                }
            }
            DMAcheck(DMA_TIMENOW);
	    break;
	case 0x05000000:
	    sprintf(str,"MMU7: PalWr: %08X=%08X",addr,val);
	    logvt->append(str);
	    GPUPAL[(addr>>1)&0x3FF]=val&65535;
	    addr+=2;
	    GPUPAL[(addr>>1)&0x3FF]=val>>16;
	case 0x06000000:
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: VRAMWr: %08X [%06X=%06X]=%08X",addr,((addr>>14)&1023)<<14,0x800000+(VRAMmap[(addr>>14)&1023]<<14),val);
            logvt->append(str);
            #endif
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val&65535; addr+=2;
	    VRAM[((VRAMmap[(addr>>14)&1023]<<14)+(addr&0x003FFF))>>1]=val>>16;
	    break;
	case 0x07000000:
	    #ifdef MMUDEBUG
            sprintf(str,"MMU7: OAMWr: %03X=%08X",addr&0x7FF,val);
            logvt->append(str);
            #endif
	    OAM[(addr&0x7FF)>>1]=val&65535; addr+=2;
	    OAM[(addr&0x7FF)>>1]=val>>16;
	    break;
	default: break;
    }
}

void MMUwrS(u32 bus, u32 addr, RAMWORD val)
{
    switch(addr&0x0F000000)
    {
        case 0x00000000: if(!(addr&0x00800000)) ITCM[(addr&0x0FFF)>>2]=val; break;
        case 0x02000000: EWRAM[(addr&0x3FFFFF)>>2]=val; break;
        case 0x03000000: 
		if(addr&0x0800000) 
			ARM7_IWRAM[(addr&0x0FFFF)>>2]=val;
		else
			IWRAM[(addr&0x07FFF)>>2]=val;
	    break;

	case 0x08000000:
	case 0x09000000:
	case 0x0A000000:
	case 0x0B000000:
	case 0x0C000000:
        case 0x0D000000: GameROM[(addr&0xFFFFFF)>>2]=val; break;
	default: break;
    }
}

